import Safe from './Safe'
import {
  CreateCallEthersContract,
  CreateProxyProps as CreateEthersProxyProps,
  EthersAdapter,
  EthersAdapterConfig,
  EthersTransactionOptions,
  EthersTransactionResult,
  MultiSendCallOnlyEthersContract,
  MultiSendEthersContract,
  SafeContractEthers,
  SafeProxyFactoryEthersContract,
  SignMessageLibEthersContract
} from './adapters/ethers'
import {
  CreateCallWeb3Contract,
  CreateProxyProps as CreateWeb3ProxyProps,
  MultiSendCallOnlyWeb3Contract,
  MultiSendWeb3Contract,
  SafeContractWeb3,
  SafeProxyFactoryWeb3Contract,
  SignMessageLibWeb3Contract,
  Web3Adapter,
  Web3AdapterConfig,
  Web3TransactionOptions,
  Web3TransactionResult
} from './adapters/web3'
import { DEFAULT_SAFE_VERSION } from './contracts/config'
import {
  getCompatibilityFallbackHandlerContract,
  getCreateCallContract,
  getMultiSendCallOnlyContract,
  getMultiSendContract,
  getProxyFactoryContract,
  getSafeContract,
  getSignMessageLibContract
} from './contracts/safeDeploymentContracts'
import {
  PREDETERMINED_SALT_NONCE,
  encodeCreateProxyWithNonce,
  encodeSetupCallData,
  predictSafeAddress
} from './contracts/utils'
import ContractManager from './managers/contractManager'
import SafeFactory, { DeploySafeProps, SafeFactoryConfig } from './safeFactory'
import {
  AddOwnerTxParams,
  ConnectSafeConfig,
  ConnectSafeConfigWithPredictedSafe,
  ConnectSafeConfigWithSafeAddress,
  ContractNetworksConfig,
  CreateTransactionProps,
  PredictedSafeProps,
  RemoveOwnerTxParams,
  SafeAccountConfig,
  SafeConfig,
  SafeConfigWithPredictedSafe,
  SafeConfigWithSafeAddress,
  SafeDeploymentConfig,
  StandardizeSafeTransactionDataProps,
  SwapOwnerTxParams
} from './types'
import { EthSafeSignature } from './utils'
import { SafeTransactionOptionalProps } from './utils/transactions/types'
import { encodeMultiSendData, standardizeSafeTransactionData } from './utils/transactions/utils'

export {
  AddOwnerTxParams,
  ConnectSafeConfig,
  ConnectSafeConfigWithPredictedSafe,
  ConnectSafeConfigWithSafeAddress,
  ContractManager,
  ContractNetworksConfig,
  CreateCallEthersContract,
  CreateCallWeb3Contract,
  CreateEthersProxyProps,
  CreateTransactionProps,
  CreateWeb3ProxyProps,
  DEFAULT_SAFE_VERSION,
  DeploySafeProps,
  EthSafeSignature,
  EthersAdapter,
  EthersAdapterConfig,
  EthersTransactionOptions,
  EthersTransactionResult,
  MultiSendCallOnlyEthersContract,
  MultiSendCallOnlyWeb3Contract,
  MultiSendEthersContract,
  MultiSendWeb3Contract,
  PREDETERMINED_SALT_NONCE,
  PredictedSafeProps,
  RemoveOwnerTxParams,
  SafeAccountConfig,
  SafeConfig,
  SafeConfigWithPredictedSafe,
  SafeConfigWithSafeAddress,
  SafeContractEthers,
  SafeContractWeb3,
  SafeDeploymentConfig,
  SafeFactory,
  SafeFactoryConfig,
  SafeProxyFactoryEthersContract,
  SafeProxyFactoryWeb3Contract,
  SafeTransactionOptionalProps,
  SignMessageLibEthersContract,
  SignMessageLibWeb3Contract,
  StandardizeSafeTransactionDataProps,
  SwapOwnerTxParams,
  Web3Adapter,
  Web3AdapterConfig,
  Web3TransactionOptions,
  Web3TransactionResult,
  encodeCreateProxyWithNonce,
  encodeMultiSendData,
  encodeSetupCallData,
  getCompatibilityFallbackHandlerContract,
  getCreateCallContract,
  getMultiSendCallOnlyContract,
  getMultiSendContract,
  getProxyFactoryContract,
  getSafeContract,
  getSignMessageLibContract,
  predictSafeAddress,
  standardizeSafeTransactionData
}

export default Safe
